//// WinCitDll.cpp : Defines the entry point for the DLL application.
//


#include <fstream>
#include "stdafx.h"
#include "Multiline.h"
#include "Recognizer.h"
#include "FeatureRecognizer.h"
#include "Vec2.h"
#include "Symbol.h"

//Testing Function
extern "C" __declspec(dllexport) int TestSquare(int input);

//Global Profile & Recognizer Related
extern "C" __declspec(dllexport) void LoadAsRecognizer(char *path);
extern "C" __declspec(dllexport) void LoadAsProfile(int *xs, int *ys, int arrsize, char*path);
extern "C" __declspec(dllexport) void LoadProfWithRec(int *xs, int *ys, int arrsize);
extern "C" __declspec(dllexport) float ProfProbOfChoice(int choicenum);
extern "C" __declspec(dllexport) int ProfNameOfChoice(int choicenum, char *buf);

//Memoryless Functions
extern "C" __declspec(dllexport) float ProbOfChoice(int choicenum, int *xs, int *ys, int arrsize, char*path);
extern "C" __declspec(dllexport) int NameOfChoice(int choicenum, int *xs, int *ys, int arrsize, char *buf, char *path);
extern "C" __declspec(dllexport) float ProbOfBest(int *xs, int *ys, int arrsize, char *path);
extern "C" __declspec(dllexport) int NameOfBest(int *xs, int *ys, int arrsize, char *buf, char *path);

//For Training
extern "C" __declspec(dllexport) void AddToTraining(int *xs, int *ys, int arrsize);
extern "C" __declspec(dllexport) int WriteTraining(char *name, char *path);
extern "C" __declspec(dllexport) void ClearTraining();

#define MAX_SAMPLES 128

Multiline Samples[MAX_SAMPLES];
int numSamples=0;
Profile globalProfile;
FeatureRecognizer *globalRecognizer=0;

BOOL APIENTRY DllMain( HANDLE hModule,
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

void LoadAsRecognizer(char *path)
{
	if(globalRecognizer)
		free(globalRecognizer);

	globalRecognizer = new FeatureRecognizer(path);
}

void LoadAsProfile(int *xs, int *ys, int arrsize, char*path)
{
	int i;
	Vec2 aVec;
	Multiline aML;
	FeatureRecognizer aFR(path);
	
	for(i=0; i < arrsize; i++)
	{
		aVec.Set(xs[i], ys[i]);
		
		if(xs[i] == 0 && ys[i] == 0)
			aML << Multiline::Jump;
		else
			aML  << aVec;
	}

	aFR.Recognize(aML, globalProfile);
}

void LoadProfWithRec(int *xs, int *ys, int arrsize)
{
	int i;
	Vec2 aVec;
	Multiline aML;

	for(i=0; i < arrsize; i++)
	{
		aVec.Set(xs[i], ys[i]);
		
		if(xs[i] == 0 && ys[i] == 0)
			aML << Multiline::Jump;
		else
			aML  << aVec;
	}

	globalRecognizer->Recognize(aML, globalProfile);
}

float ProfProbOfChoice(int choicenum)
{
	return globalProfile.GetProb(choicenum);	
}

int ProfNameOfChoice(int choicenum, char *buf)
{
	strcpy(buf, globalProfile.GetSymbol(choicenum)->Name());
	return 0;
}

float ProbOfChoice(int choicenum, int *xs, int *ys, int arrsize, char *path)
{
	int i;
	Vec2 aVec;
	Multiline aML;
	Profile aProfile;
	FeatureRecognizer aFR(path);
	
	for(i=0; i < arrsize; i++)
	{
		aVec.Set(xs[i], ys[i]);
		
		if(xs[i] == 0 && ys[i] == 0)
			aML << Multiline::Jump;
		else
			aML  << aVec;
	}

	if(aFR.Recognize(aML, aProfile))
		return aProfile.GetProb(choicenum);
	return -1; //As error code
}

int NameOfChoice(int choicenum, int *xs, int *ys, int arrsize, char *buf, char *path)
{
	int i;
	Vec2 aVec;
	Multiline aML;
	Profile aProfile;
	FeatureRecognizer aFR(path);
	Symbol *nthPick;
	
	for(i=0; i < arrsize; i++)
	{
		aVec.Set(xs[i], ys[i]);
		
		if(xs[i] == 0 && ys[i] == 0)
			aML << Multiline::Jump;
		else
			aML  << aVec;
	}

	if(aFR.Recognize(aML, aProfile))
	{
		nthPick = aProfile.GetSymbol(choicenum);
		strcpy(buf, nthPick->Name());
		return 0;
	}
	return -1; //As error code
}

float ProbOfBest(int *xs, int *ys, int arrsize, char *path)
{
	return ProbOfChoice(0, xs, ys, arrsize, path);
}

int NameOfBest(int *xs, int *ys, int arrsize, char *buf, char *path)
{
	return NameOfChoice(0, xs, ys, arrsize, buf, path);
}

int TestSquare(int input)
{
	return input * input;
}

void AddToTraining(int *xs, int *ys, int arrsize)
{
	int i;
	Vec2 aVec;
	Multiline aML;
	
	if(numSamples >= MAX_SAMPLES)
		return;

	for(i=0; i < arrsize; i++)
	{
		aVec.Set(xs[i], ys[i]);
		
		if(xs[i] == 0 && ys[i] == 0)
			aML << Multiline::Jump;
		else
			aML  << aVec;
	}
	
	Samples[numSamples] = aML;
	numSamples ++;
}

int WriteTraining(char *name, char *path)
{
	int i;
	char arr[256];
	fstream afs;

	strcpy(arr, path);
	if(name[0] >= 'a' && name[0] <= 'z')
		strcat(arr,"_");
	strcat(arr, name);
	strcat(arr, ".0");

	afs.open(arr, fstream::out);
	afs << "[Training Data: Version 3]" << endl;
	afs << numSamples << endl;
	afs << name <<endl;
	
	for(i=0; i < numSamples; i++)
		afs << Samples[i];
	afs.flush();
	afs.close();
	return 0;
}

void ClearTraining()
{
	numSamples = 0;
}