/***************************************************************************
* BasicGrouper.h                                                           *
*                                                                          *
* Class definitions for a straight-forward stroke grouping algorithm.      *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       10/20/99  arvo    Initial coding.                                  *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __BASIC_GROUPER_INCLUDED__
#define __BASIC_GROUPER_INCLUDED__

#include "../util/Multiline.h"
#include "../rec/Recognizer.h"
#include "../rec/Grouper.h"

static const int MaxLookahead = 3 * MaxSymbolStrokes;

class SearchStatus {
    public:
      SearchStatus() { Init(); }
     ~SearchStatus() {}
      void Init() { min = 0.0; count = 0; next = 0; }
      float min;
      int   count;
      int   next;
    };

class BasicGrouper : public Grouper {

    public:
      BasicGrouper();
     ~BasicGrouper() {}
      void SetLookahead( int n ) { lookahead = n; }

    public: // These override the virtual functions.
      void    Clear();
      void    SetRecognizer( Recognizer &recognizer ) { rec = &recognizer; }
      void    operator<<( const Multiline & );
      Symbol *GetCharacter( Multiline &, int force = 0 );

    private:
      Multiline Remove( int n );
      float     GetProb( int from_index, int to_index );
      Symbol   *GetSymb( int from_index, int to_index );
      float     FindBestGrouping( int i, int j, float min, SearchStatus & );

    private:
      Recognizer *rec;
      Multiline strokes[ MaxLookahead ];
      float prob[ MaxLookahead ][ MaxLookahead ];
      int   num_strokes;
      int   lookahead;

    };

#endif





