/***************************************************************************
* LineMorph.h                                                              *
*                                                                          *
* This module defines all the functions that are called from Tcl/Tk via    *
* the SendMessage and Dispatch mechanisms.                                 *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       10/09/99    arvo    Integrated into Train program.                 *
*       07/06/97    arvo    Initial coding.                                *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __LINEMORPH_INCLUDED__
#define __LINEMORPH_INCLUDED__

#include "../util/Multiline.h"

// Both multilines are modified by splitting, refining, reordering
// and reparameterizing (flipping) their components so that
//    1) they have the same number of components
//    2) corresponding components have the same number of points
//    3) total energy of the matching is minimized.
//
extern double LineMorphSetup(  // Returns energy
    Multiline &P, 
    Multiline &Q 
    );

extern double LineMorphSetup2(  // Returns energy
    Multiline &P, 
    Multiline &Q 
    );

extern int BreakSharpestBend( 
    Multiline &P, 
    double max_angle = 0.75  // 0 = 180 degree fold, 1 = no bend.
    );

// One-to-one linear interpolation of multilines.  This will only
// work if P and Q have the same number of components, and
// corresponding components have the same number of points.

extern Multiline Interpolate( 
    double t,            // Parameter in [0,1].
    const Multiline &P, // Multiline at t = 0.
    const Multiline &Q  // Multiline at t = 1.
    );

extern void SimpleMorphSetup( 
    Multiline &P, 
    Multiline &Q 
    );

#endif



