/***************************************************************************
* Symbol.C																   *
*																		   *
*	Changes:															   *
*																		   *
*		06/02/98	arvo	  Initial coding.							   *
*																		   *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo										   *
*																		   *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html	   *
*																		   *
* This program is distributed in the hope that it will be useful, but	   *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more	   *
* details.																   *
*																		   *
***************************************************************************/
#include "stdafx.h"

#include <stdio.h>
#include <string.h>
#include "../rec/Symbol.h"

// This is a (hopefully) unique marker for superscript and
// subscript positions that have not been explicitly set in
// the ".font" file.
const Vec2 NotSupplied( -100, -100 );

Symbol::Symbol( const char *n, const char *q ) : NameTableItem( n, q )
{
	box 	= Box2d::Null;
	sub_pos = NotSupplied;
	exp_pos = NotSupplied;
	width	= 0.0;
}

Symbol::~Symbol()
{
	stroke .Clear();
	outline.Clear();
}

void Symbol::SetStroke( const Multiline &M ) 
{ 
	stroke = M; 
	box << M.Box();
}

void Symbol::SetOutline( const Multiline &M ) 
{ 
	outline = M; 
	box << M.Box();
}

void Symbol::SetMesh( const Multiline &M ) 
{ 
	mesh = M; 
}

void Symbol::SetExpPos( const Vec2 &p ) 
{ 
	exp_pos = p;
}

void Symbol::SetSubPos( const Vec2 &p ) 
{ 
	sub_pos = p; 
}

void Symbol::SetWidth( double w ) 
{ 
	width = w; 
}

void Symbol::ComputeMissingFields()
{
	// Note that Y is pointing down!  The coordinates have
	// been flipped by this time!
	if( exp_pos == NotSupplied ) 
		exp_pos = Vec2( box.Xmax(), box.Ymin() );
	
	if( sub_pos == NotSupplied ) 
		sub_pos = Vec2( box.Xmax(), box.Ymax() );
}
