/***************************************************************************
* Args.C																   *
*																		   *
* The CommandLine class is used for accessing and parsing command line	   *
* arguments.  This class need not be instanced, since all the data and	   *
* methods are static.													   *
*																		   *
*	HISTORY 															   *
*		09/02/98	arvo	Initial coding. 							   *
*																		   *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo										   *
*																		   *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html	   *
*																		   *
* This program is distributed in the hope that it will be useful, but	   *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more	   *
* details.																   *
*																		   *
***************************************************************************/
//#include <iostream.h>
#include <string.h>
#include "../util/Args.h"

int    CommandLine::argc = 0;
char **CommandLine::argv = NULL;

// Store the "argc" and "argv" arguments passed to the main program in
// static variables assoviated with this class, so that any other module
// can access the command-line arguments.
CommandLine::CommandLine( int _argc, char *_argv[] )
{
	argc = _argc;
	argv = _argv;
}

void CommandLine::Set( int _argc, char *_argv[] )
{
	argc = _argc;
	argv = _argv;
}

inline int is_switch( const char *arg )
{
	return arg[0] == '-';
}

static int starts_with( const char *str, const char *pattern )
{
	while( *pattern )
		if( *str++ != *pattern++ ) return 0;
		return 1;
}

static int substring( const char *pattern, const char *str )
{
	while( *str )
	{
		if( starts_with( str, pattern ) ) return 1;
		str++;
	}
	return 0;
}

const char *CommandLine::GetArg( int index )
{
	if( argc == 0 || argv == NULL ) return NULL;
	if( index >= argc ) index = argc - 1;
	if( index < 0	  ) index = 0;
	return argv[ index ];
}

const char *CommandLine::GetArg( const char *flag, int index )
{
	// First, find a switch argument that contains the given flag.
	for( int i = 1; i < argc; i++ )
	{
		const char *arg = argv[i];
		if( is_switch( arg ) && substring( flag, arg + 1 ) )
		{
			int k = 0;
			// Now find the "index"th argument following the switch.
			for( int j = i + 1; j < argc; j++ )
			{
				arg = argv[j];
				if( is_switch( arg ) ) return NULL;
				if( k++ == index ) return arg;
			}
			return NULL;
		}
	}
	return NULL;
}

int CommandLine::GetSwitch( const char *flag, const char *match_arg )
{
	for( int i = 1; i < argc; i++ )
	{
		const char *arg = argv[i];
		if( is_switch( arg ) && substring( flag, arg + 1 ) ) 
		{
			// If "match_arg" is null, we were just looking for the existence
			// of the flag, regardless of what follows it.
			if( match_arg == NULL ) return 1;
			// Otherwise, check that the arg that immediately follows exactly 
			// matches the string "match_arg".
			if( i + 1 >= argc ) return 0;
			return strcmp( argv[i+1], match_arg ) == 0;
		}
	}
	return 0;
}

#include "stdafx.h"
