/***************************************************************************
* List.h                                                                   *
*                                                                          *
* Base class for objects that may be inserted into a list.                 *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       11/04/98    arvo    Copied from Queue.h                            *
*       06/22/97    arvo    Initial coding.                                *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __LIST_INCLUDED__
#define __LIST_INCLUDED__

#include <fstream>
using namespace std;

class List;     // List of objects derived from ListItem.
class ListItem; // Base class for elements in a List.

class ListItem {

    public:
        ListItem();
        virtual  ~ListItem ();
        List     *Owner    () const { return owner; }
        ListItem *Next     () const { return next ; }
        ListItem *Back     () const { return back ; }

    private:
        ListItem *back;  // The previous item in the List.
        ListItem *next;  // The next item in the List.
        List     *owner; // The List that owns this item.

    friend class List;
    };


class List {

    public:
        List();
        virtual   ~List();
        List      &operator<<( ListItem * ); // Insert an item.
        ListItem  *operator>>( ListItem * ); // Remove the given item.
        void       Clear();                  // Destroy all entries.
        int        Empty() const { return first == NULL; }
        ListItem  *First() const { return first; }
        ListItem  *Last () const { return last; }
        int        NumEntries() const { return size; }

    private:
        ListItem *first;
        ListItem *last;
        int       size;

    };


#endif
